from argparse import ArgumentParser
from dodo_commands.framework import Dodo, CommandError
from dodo_commands.framework.config import CommandPath, ConfigIO
import os
import sys


def _default_commands_basename():
    return 'dodo_%s_commands' % Dodo.get_config('/ROOT/project_name')


def _default_commands_dir():
    return os.path.normpath(os.path.join(
        Dodo.get_config('/ROOT/shared_config_dir'),
        '..',
        _default_commands_basename()
    ))


def _args():
    parser = ArgumentParser(description="Creates a new Dodo command.")
    parser.add_argument('name')
    parser.add_argument(
        '--create-commands-dir',
        action='store_true',
        help="Create a directory for storing commands in the shared_config_dir",
    )
    parser.add_argument(
        '--next-to',
        help='Create the new command at the location of this command')
    args = Dodo.parse_args(parser)

    if not args.create_commands_dir and not args.next_to:
        raise CommandError('Either --create_commands_dir or --next-to is required')

    return args


script = """import os
from argparse import ArgumentParser
from dodo_commands.framework import Dodo, ConfigArg, CommandError


def _args():
    parser = ArgumentParser(
        description='A new command that runs in the project\\\'s "res" directory'
    )

    # Add arguments to the parser here
    parser.add_argument('name')

    # Parse the arguments.
    # Optionally, use `config_args` to include additional arguments whose values
    # come either from the configuration file or from the command line (see docs).
    args = Dodo.parse_args(parser, config_args=[
        ConfigArg('/ROOT/res_dir', '--cwd')
    ])

    # Insert additional arguments after parsing
    args.project_name = Dodo.get_config('/ROOT/project_name')

    # Raise an error if something is not right
    if not os.path.exists(args.cwd):
        raise CommandError('Not found: %s' % args.cwd)

    return args


# Use safe=False if the script makes changes other than through Dodo.run
if Dodo.is_main(__name__, safe=True):
    args = _args()
    Dodo.run(['echo', args.name, args.project_name], cwd=args.cwd)
"""

if Dodo.is_main(__name__, safe='--create-commands-dir' not in sys.argv):
    args = _args()

    if args.create_commands_dir:
        dest_path = os.path.join(_default_commands_dir(), args.name + ".py")
        command_path = CommandPath(Dodo.get_config())

        if not os.path.exists(_default_commands_dir()):
            Dodo.run(['mkdir', '-p', _default_commands_dir()])

        init_py = os.path.join(_default_commands_dir(), '__init__.py')
        if not os.path.exists(init_py):
            Dodo.run(['touch', init_py])

        if not [
            x for x in command_path.items
            if os.path.normpath(x) == _default_commands_dir()
        ]:
            config = ConfigIO().load(load_layers=False)
            scd = config['ROOT']['shared_config_dir']
            config['ROOT']['command_path'].append(
                os.path.join(scd[:scd.rfind('/')], _default_commands_basename())
            )
            ConfigIO().save(config)

    if args.next_to:
        dest_path = None
        command_path = CommandPath(Dodo.get_config())
        for item in command_path.items:
            script_path = os.path.join(item, args.next_to + ".py")
            if os.path.exists(script_path):
                dest_path = os.path.join(item, args.name + ".py")
        if not dest_path:
            raise CommandError("Script not found: %s" % args.next_to)

    if os.path.exists(dest_path):
        raise CommandError("Destination already exists: %s" % dest_path)

    with open(dest_path, "w") as f:
        f.write(script)
    print(dest_path)



from argparse import ArgumentParser
from dodo_commands.framework import Dodo
import sys


def _args():
    parser = ArgumentParser(description='Print the command line to cd to'
                            ' a folder inside the project folder.')
    parser.add_argument(
        'to',
        nargs='?',
        help=('cd to /ROOT/<to>_dir. For example: dodo cd src ' +
              'cds to the value of /ROOT/src_dir.'))
    args = Dodo.parse_args(parser)
    args.path = Dodo.get_config("/ROOT/%s_dir" % (args.to or 'project'))
    return args


if Dodo.is_main(__name__):
    args = _args()
    sys.stdout.write("cd %s\n" % args.path)



"""Script for activating/creating a project in the projects dir."""
from argparse import ArgumentParser
from dodo_commands.framework import Dodo, CommandError
from dodo_commands.framework.config import (load_global_config_parser,
                                            write_global_config_parser, Paths)
from dodo_commands.framework.util import is_windows, symlink
import os
import shutil
import sys
import ruamel.yaml
from plumbum import local
import dodo_commands


def _make_executable(script_filename):
    st = os.stat(script_filename)
    os.chmod(script_filename, st.st_mode | 0o111)


def dodo_template(python_path):
    return """#!{python_path}
import os
from os.path import abspath, realpath, dirname
import sys
from dodo_commands.framework import execute_from_command_line

if __name__ == "__main__":
    exe_dir = dirname(realpath(abspath(__file__)))
    if not exe_dir.endswith("env{sep}{bin}"):
        sys.stderr.write(
            'Error: this script must be run from the env{sep}{bin} directory')
        sys.exit(1)

    os.environ["DODO_COMMANDS_PROJECT_DIR"] = dirname(
        dirname(dirname(exe_dir)))
    execute_from_command_line(sys.argv)
""".format(
        python_path=python_path,
        bin='Scripts' if is_windows() else 'bin',
        sep=os.sep)


def dodo_template_bat(python_path, dodo_path):
    return """
@ECHO OFF
{python_path} {dodo_path} %*
""".format(
        python_path=python_path, dodo_path=dodo_path)


class Activator:
    """Creates and activates projects."""

    def _virtual_env_filename(self, basename):
        return os.path.join(self.paths.virtual_env_bin_dir(), basename)

    def _create_virtual_env(self):
        """Install a virtual env."""
        try:
            local["virtualenv"]('--version')
        except:
            raise CommandError("Could not find virtualenv, please install it first.")

        local["virtualenv"]("-p",
                            self.config.get("settings", "python_interpreter"),
                            self.paths.virtual_env_dir(), "--prompt", self.project)

        local[self.paths.pip()](
            "install",
            "ansimarkup",
            "argcomplete",
            "parsimonious",
            "plumbum",
            "ruamel.yaml",
            "semantic_version",
            "six",
        )

        symlink(
            os.path.dirname(dodo_commands.__file__),
            os.path.join(self.paths.site_packages_dir(), "dodo_commands"))

    def _create_dodo_script(self):
        """Install the dodo entry point script."""
        dodo_file = self._virtual_env_filename('dodo')
        python_path = os.path.join(self.paths.virtual_env_bin_dir(), 'python')
        with open(dodo_file, "w") as of:
            of.write(dodo_template(python_path))
        _make_executable(dodo_file)

        if is_windows():
            dd_file = self._virtual_env_filename('dd.bat')
            with open(dd_file, "w") as of:
                of.write(dodo_template_bat(python_path, dodo_file))

    def _create_res_dir(self):
        """Install the dir with dodo_commands resources."""
        res_dir = self.paths.res_dir()
        os.makedirs(res_dir)
        config_filename = os.path.join(res_dir, "config.yaml")
        default_config = {
            'ROOT': {
                'command_path':
                [os.path.join(self.paths.default_commands_dir(), '*')],
                'version':
                '1.0.0'
            }
        }
        with open(config_filename, "w") as f:
            f.write(ruamel.yaml.round_trip_dump(default_config))

    def _dodo_commands_sub_dir(self):
        return os.path.join(self.paths.project_dir(), "dodo_commands")

    def _create_project(self):
        project_dir = self.paths.project_dir()
        remove_project_dir_on_failure = not os.path.exists(project_dir)

        try:
            self._create_res_dir()
            self._create_virtual_env()
            self._create_dodo_script()
        except:
            if remove_project_dir_on_failure:
                if os.path.exists(project_dir):
                    shutil.rmtree(project_dir)
            elif os.path.exists(self._dodo_commands_sub_dir()):
                shutil.rmtree(self._dodo_commands_sub_dir())
            self._report('\n')
            raise

    def _report(self, x):
        sys.stderr.write(x)
        sys.stderr.flush()

    def _config_get(self, section, key, default=""):
        return (self.config.get(section, key)
                if self.config.has_option(section, key) else default)

    def run(self, project, latest, create):
        """Activate or create a project in the projects dir."""
        self.config = load_global_config_parser()

        if not self.config.has_section('recent'):
            self.config.add_section('recent')

        latest_project = self._config_get("recent", "latest_project")
        previous_project = self._config_get("recent", "previous_project")

        if project == '-':
            project = previous_project
            if not project:
                raise CommandError("There is no previous project")
        self.project = project

        if latest:
            if self.project:
                self._report(
                    "Options --latest and <project> are mutually exclusive\n")
                return
            self.project = latest_project
            if not self.project:
                self._report("There is no latest project\n")
                return

        self.paths = Paths(
            project_dir=os.path.expanduser(
                os.path.join(
                    self.config.get("settings", "projects_dir"),
                    self.project)))

        if create:
            project_dir = self.paths.project_dir()

            if os.path.exists(self._dodo_commands_sub_dir()):
                self._report(
                    "Project already exists: %s\n" % project_dir)
                return False

            self._report(
                "Creating project at location %s ..." % project_dir)
            self._create_project()
            self._report(" done\n")
        elif not os.path.exists(self.paths.project_dir()):
            raise CommandError(
                'Project not found: %s. Use the --create flag to create it\n' %
                self.paths.project_dir())

        if self.project != latest_project:
            self.config.set("recent", "previous_project", latest_project)
            self.config.set("recent", "latest_project", self.project)
            write_global_config_parser(self.config)

        sys.stdout.write(
            "source %s\n" % self._virtual_env_filename("activate"))


def _args():
    parser = ArgumentParser()
    parser.add_argument('project', nargs='?')

    group = parser.add_mutually_exclusive_group()
    group.add_argument('--latest', action="store_true")
    group.add_argument('--create', action="store_true")
    args = Dodo.parse_args(parser)
    return args


if Dodo.is_main(__name__, safe=False):
    args = _args()
    if not args.project and not args.latest:
        raise CommandError("No project was specified")
    Activator().run(args.project, args.latest, args.create)



from argparse import ArgumentParser
from dodo_commands.framework import Dodo
from dodo_commands.framework.config import Paths
import os


def _args():
    parser = ArgumentParser(
        description=(
            "Writes (or removes) a small script that activates the latest " +
            "Dodo Commands project"))
    parser.add_argument('status', choices=['on', 'off'])
    return Dodo.parse_args(parser)


if Dodo.is_main(__name__, safe=False):
    args = _args()

    script = os.path.join(Paths().home_dir(), '.dodo_commands_autostart')
    if args.status == "on" and not os.path.exists(script):
        with open(script, "w") as f:
            f.write("$(dodo activate --latest) && dodo check-version --dodo --config\n")
    if args.status == "off" and os.path.exists(script):
        os.unlink(script)



from argparse import ArgumentParser
from dodo_commands.framework import Dodo
from plumbum.cmd import docker
from six.moves import input as raw_input
import sys


def _args():
    parser = ArgumentParser()
    args = Dodo.parse_args(parser)
    return args


def _containers():
    result = []
    for line in docker("ps", "--format",
                       "{{.ID}} {{.Names}} {{.Image}}").split('\n'):
        if line:
            cid, name, image = line.split()
            result.append(dict(name=name, cid=cid, image=image))
    return result


if Dodo.is_main(__name__):
    args = _args()
    containers = _containers()

    print("0 - exit")
    for idx, container in enumerate(containers):
        print("%d - %s" % (idx + 1, container['name']))

    print("\nSelect a container: ")
    choice = int(raw_input()) - 1

    if choice == -1:
        sys.exit(0)

    container = containers[choice]

    Dodo.run([
        'docker',
        'commit',
        container['cid'],
        container['image'],
    ], )



from dodo_commands.framework import Dodo


if Dodo.is_main(__name__, safe=True):
    print(
        'sudo register-python-argcomplete dodo > ' +
        '/etc/bash_completion.d/dodo_autocomplete.sh'
    )



from argparse import ArgumentParser
from dodo_commands.framework import Dodo
from dodo_commands.framework.config import ConfigIO
from plumbum.cmd import docker


def _args():
    parser = ArgumentParser()
    parser.add_argument(
        'container_type',
        choices=Dodo.get_config('/DOCKER/container_types', {}).keys())
    parser.add_argument('name')
    args = Dodo.parse_args(parser)
    args.dirs = Dodo.get_config(
        "/DOCKER/container_types/%s/dirs" % args.container_type)
    args.image = Dodo.get_config(
        "/DOCKER/container_types/%s/image" % args.container_type)
    return args


if Dodo.is_main(__name__, safe=False):
    args = _args()
    existing = docker("ps", "-a", "--quiet", "--filter", "name=" + args.name)
    if not existing:
        cmd_args = [
            "docker",
            "create",
            "--name",
            args.name,
        ]

        for path in args.dirs:
            cmd_args.extend(["-v", path])

        cmd_args.append(args.image)
        Dodo.run(cmd_args)

    config = ConfigIO().load(load_layers=False)
    config['DOCKER'].setdefault('containers',
                                {})[args.container_type] = args.name
    ConfigIO().save(config)



from setuptools import setup

setup(
    name='dodo_commands',
    version='0.21.0',
    description=
    'Project-aware development environments, inspired by django-manage',
    url='https://github.com/mnieber/dodo_commands',
    download_url='https://github.com/mnieber/dodo_commands/tarball/0.21.0',
    author='Maarten Nieber',
    author_email='hallomaarten@yahoo.com',
    license='MIT',
    packages=[
        'dodo_commands',
        'dodo_commands.framework',
        'dodo_commands.dodo_system_commands',
    ],
    package_data={
        'dodo_commands': [
            'extra/__init__.py',
            'extra/dodo_standard_commands/*.py',
            'extra/dodo_standard_commands/*.meta',
            'extra/dodo_standard_commands/decorators/*.py',
        ]
    },
    entry_points={'console_scripts': [
        'dodo=dodo_commands.dodo:main',
    ]},
    install_requires=[
        'argcomplete',
        'plumbum',
        'ruamel.yaml',
        'six',
    ],
    zip_safe=False)



from argparse import ArgumentParser
from dodo_commands.framework import Dodo
from dodo_standard_commands.decorators.docker import (Decorator as
                                                      DockerDecorator)


def _args():
    parser = ArgumentParser(
        description='Opens a bash shell in the docker container.')
    parser.add_argument(
        'service',
        choices=Dodo.get_config('/DOCKER/options', {}).keys(),
        help=("Use this key to look up the docker options in /DOCKER/options"))
    parser.add_argument(
        '--image',
        choices=Dodo.get_config('/DOCKER/images', {}).keys(),
        help=("Use the docker image stored under this key in /DOCKER/images"))
    parser.add_argument(
        '--image-name', help=("Use the docker image with this name"))
    parser.add_argument(
        '--name', help=("Override the name of the started docker container"))
    parser.add_argument('--command')
    args = Dodo.parse_args(parser)
    return args


if Dodo.is_main(__name__):
    args = _args()

    docker_options = DockerDecorator.merged_options(Dodo.get_config,
                                                    args.service)

    if args.image:
        docker_options['image'] = Dodo.get_config(
            '/DOCKER/images/%s/image' % args.image, args.image)
    elif args.image_name:
        docker_options['image'] = args.image_name

    if args.name:
        docker_options['name'] = args.name
    else:
        docker_options['name'] = args.service

    Dodo.get_config('/DOCKER')['options'] = {Dodo.command_name: docker_options}
    Dodo.run(
        ["/bin/bash"] + (["-c", args.command] if args.command else []),
        cwd=docker_options.get("cwd", "/"))



from dodo_commands.framework import Dodo, CommandError
from dodo_commands.framework.util import remove_trailing_dashes
import argparse
import os


def _create_get_config(local_vars):
    """
    Returns a get_config function that interpolates local_vars.
    For example:

    gc = Dodo._create_get_config(dict(user='sam'))
    tmp_dir = gc('/USERS/{user}/tmp_dir', default='/tmp')
    """

    def get_config(key, default=None):
        return Dodo.get_config(key.format(**local_vars), default)

    return get_config


def _dodo_args():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        'name',
        choices=Dodo.get_config('/DOCKER/images').keys(),
        help='Key to look up in /DOCKER/images')
    parser.add_argument(
        'build_args',
        nargs=argparse.REMAINDER,
        help='Extra args to pass to docker build')
    return Dodo.parse_args(parser)


def _convert_dodo_args():
    dodo_args = _dodo_args()
    gc = _create_get_config(dodo_args.__dict__)
    args = argparse.Namespace()
    args.build_dir = gc(key='/DOCKER/images/{name}/build_dir', default='.')
    args.docker_file = gc(
        key='/DOCKER/images/{name}/docker_file', default='Dockerfile')
    args.extra_dirs = gc(key='/DOCKER/images/{name}/extra_dirs', default=[])
    args.docker_image = gc(key='/DOCKER/images/{name}/image')
    args.build_args = dodo_args.build_args
    return args


def _args():
    parser = argparse.ArgumentParser()
    parser.add_argument('--build-dir', default='.')
    parser.add_argument('--docker-filename', default='Dockerfile')
    parser.add_argument('--extra_dir', action='append', dest='extra_dirs')
    parser.add_argument('docker_image')
    parser.add_argument('build_args', nargs=argparse.REMAINDER)
    return Dodo.parse_args(parser)


def _copy_extra_dirs(local_dir, extra_dirs):
    # todo: convert extra dirs config mapping into a list of key=val
    for extra_dir in (extra_dirs or []):
        extra_dir_name, extra_dir_path = extra_dir.split('=')
        local_path = os.path.join(local_dir, extra_dir_name)
        if os.path.exists(local_path):
            raise CommandError("Cannot copy to existing path: " + local_path)
        if not os.path.exists(extra_dir_path):
            raise CommandError("Cannot copy from non-existing path: " +
                               extra_dir_path)
        Dodo.run(['cp', '-rf', extra_dir_path, local_path])


def _remove_extra_dirs(local_dir, extra_dirs):
    for extra_dir in (extra_dirs or []):
        extra_dir_name, extra_dir_path = extra_dir.split('=')
        local_path = os.path.join(local_dir, extra_dir_name)
        Dodo.run(['rm', "rm", "-rf", local_path])


args = (_convert_dodo_args() if Dodo.command_name else _args())

if Dodo.is_main(__name__, safe=len(args.extra_dirs) == 0):
    _copy_extra_dirs(args.build_dir, args.extra_dirs)

    try:
        Dodo.run(
            [
                "docker",
                "build",
                "-t",
                args.docker_image,
                "-f",
                args.docker_file,
            ] + remove_trailing_dashes(args.build_args) + [
                ".",
            ],
            cwd=args.build_dir)
    finally:
        _remove_extra_dirs(args.build_dir, args.extra_dirs)
